@@ -2,7 +2,7 @@
  * Gnome Chemistry Utils
  * object.cc 
  *
- * Copyright (C) 2002-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,7 +23,9 @@
 #include "config.h"
 #include "object.h"
 #include "objprops.h"
+#include "dialog.h"
 #include "document.h"
+#include <glib/gi18n.h>
 #include <string>
 #include <iostream>
 #include <vector>
@@ -83,8 +85,20 @@ Object::~Object ()
 			m_Children.erase ((*i).first);
 		}
 	}
+	while (!m_Links.empty ())
+		Unlink (*(m_Links.begin ()));
 }
 
+void Object::Clear ()
+{
+	map<string, Object*>::iterator i;
+	while (!m_Children.empty ()) {
+		i = m_Children.begin ();
+		(*i).second->m_Parent = NULL;
+		delete (*i).second;
+		m_Children.erase ((*i).first);
+	}
+}
 
 void Object::SetId (gchar const *Id)
 {
@@ -157,7 +171,7 @@ void Object::AddChild (Object* object)
 	if (object->m_Id == NULL) {
 		int i = 1;
 		char szId[16];
-		while (snprintf (szId, sizeof(szId), "o%d", i++), pDoc->GetDescendant (szId) != NULL);
+		while (snprintf (szId, sizeof(szId), "o%d", i++), pDoc->GetDescendant (szId) != NULL) ;
 		object->m_Id = g_strdup (szId);
 	} else {
 		Object* o = pDoc->RealGetDescendant (object->m_Id);
@@ -387,36 +401,38 @@ bool Object::BuildContextualMenu (GtkUIManager *UIManager, Object *object, doubl
 	return result | ((m_Parent)? m_Parent->BuildContextualMenu (UIManager, object, x, y): false);
 }
 
-void Object::Add (GtkWidget* w) const
-{
-	map<string, Object*>::const_iterator i;
-	Object const *p = GetFirstChild (i);
-	while (p) {
-		p->Add (w);
-		p = GetNextChild (i);
-	}
-}
-
-void Object::Update (GtkWidget* w) const
-{
-	map<string, Object*>::const_iterator i, end = m_Children.end ();
-	for (i = m_Children.begin (); i != end; i++)
-		(*i).second->Update (w);
-}
-
-void Object::SetSelected (GtkWidget* w, int state)
-{
-	map<string, Object*>::iterator i, end = m_Children.end ();
-	for (i = m_Children.begin (); i != end; i++)
-		(*i).second->SetSelected (w, state);
-}
-
-Object* Object::GetAtomAt (double x, double y, double z)
+char const *Object::HasPropertiesDialog () const
 {
 	return NULL;
 }
 
-bool Object::Build (list<Object*>& Children) throw (invalid_argument)
+Dialog *Object::BuildPropertiesDialog ()
+{
+	return NULL;
+}
+
+void Object::ShowPropertiesDialog ()
+{
+	const char *dlg_name = HasPropertiesDialog ();
+	if (!dlg_name)
+		return;
+	DialogOwner *owner = dynamic_cast <DialogOwner *> (this);
+	if (!owner)
+		return;
+	Dialog *dlg = owner->GetDialog (dlg_name);
+	if (!dlg)
+		dlg = BuildPropertiesDialog ();
+	if (dlg)
+		dlg->Present ();
+	
+}
+
+Object* Object::GetAtomAt (G_GNUC_UNUSED double x, G_GNUC_UNUSED double y, G_GNUC_UNUSED double z)
+{
+	return NULL;
+}
+
+bool Object::Build (G_GNUC_UNUSED list<Object*>& Children) throw (invalid_argument)
 {
 	return false;
 }
@@ -451,6 +467,16 @@ TypeId Object::AddType (string TypeName, Object* (*Create) (), TypeId id)
 	return typedesc.Id;
 }
 
+void Object::AddAlias (TypeId id, std::string TypeName)
+{
+	if (id > TypeNames.size ())
+		return;
+	string &name = TypeNames[id];
+	if (name.length () == 0)
+		return;
+	Types[TypeName] = Types[name];
+}
+
 Object* Object::CreateObject (const string& TypeName, Object* parent)
 {
 	TypeDesc& typedesc = Types[TypeName];
@@ -587,7 +613,7 @@ void Object::EmitSignal (SignalId Signal)
 	}
 }
 
-bool Object::OnSignal (SignalId Signal, Object *Child)
+bool Object::OnSignal (G_GNUC_UNUSED SignalId Signal, G_GNUC_UNUSED Object *Child)
 {
 	return true;
 }
@@ -608,13 +634,18 @@ Object* Object::GetNextLink (set<Object*>::iterator& i)
 	return *i;
 }
 
+void Object::Link (Object *object)
+{
+	m_Links.insert (object);
+}
+
 void Object::Unlink (Object *object)
 {
 	m_Links.erase (object);
-	OnUnlink (object);
+	object->OnUnlink (this);
 }
 
-void Object::OnUnlink (Object *object)
+void Object::OnUnlink (G_GNUC_UNUSED Object *object)
 {
 }
 
@@ -632,7 +663,7 @@ void Object::AddMenuCallback (TypeId Id, BuildMenuCb cb)
 	typedesc.MenuCbs.push_back (cb);
 }
 
-bool Object::SetProperty (unsigned property, char const *value)
+bool Object::SetProperty (G_GNUC_UNUSED unsigned property, G_GNUC_UNUSED char const *value)
 {
 	return true;
 }
@@ -662,4 +693,14 @@ void Object::SetDirty (bool dirty)
 	}
 }
 
+std::string Object::Identity ()
+{
+	return Name () + " " + GetId ();
+}
+
+std::string Object::Name ()
+{
+	return _("Object");
+}
+
 }	//	namespace gcu
